/******************************************************************
Copyright (C) 2010, Kengo NAGASHIMA.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation, either version 3 of the License,
or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>
******************************************************************/

/*****************************************************************
  Author: Kengo NAGASHIMA
  Version: 2.0.0
  Last Updated Date: 03/02/2010
  Recommended Environment:
      Windows XP 32 bit, SAS 9.1.3,
      set up SAS output fint size = 12pt
*****************************************************************/

/*****************************************************************
  Make graph datasets for Kaplan-Meier plot
    data: a input dataset name
    time: a survival time variable name
    stra: a stratum variable name
    censor: a censor variable name
    censorv: censored variable code
    out: output graph dataset name
    anno: output annotate dataset name
    afont: no. at risk text font
    censEXT: 0 = none
             1 = censor extension (needle)
    cHeight: extended censor height
    cWidth: extended censor width
    atrisk: 0 = none
            1 = no. at risk
    atriskorder: no. at risk order
    Base: no. at risk base position
    Step: no. at risk interval step size
    Size: no. at risk font size
    Label: no. at risk label
*/
%macro km_data(
  data, time, stra, censor, censorv,
  out = graph, anno = anno,
  afont = "'Times New Roman'",
  censEXT = 0, cHeight = 0.025, cWidth = 5,
  atrisk = 1, atriskorder = 0 to 0,
  Base = 0, Step = 4, Size = 3, Label = "No. at risk"
);
ods exclude all;
proc lifetest data = &data. outsurv = &out.;
  time &time. * &censor.(&censorv.);
  strata &stra.;
  ods output
    CensoredSummary = CS ProductLimitEstimates = PLE Quartiles = QEST;
run;
ods select all;
proc sort data = &out.; by &stra.;
data &out.;
  set &out.; retain CSDF 0;
  if Survival = . then Survival = CSDF;
  else CSDF = Survival;
  drop &stra. SDF_LCL SDF_UCL;
data &out.;
  set
    &out.(where=(_CENSOR_ ^= 1) drop=CSDF)
    &out.(where=(_CENSOR_ = 1));
  drop _CENSOR_;
proc sort data = &out.; by &time.;
data _null_;
  set CS; call symput("StraN", cat(_N_-1));
data &out.;
  merge
  %do i = 1 %to &StraN.;
    &out.(where=(STRATUM=&i.) rename=(Survival=SV&i.) drop=CSDF)
    &out.(where=(STRATUM=&i. & CSDF&i.^=.) rename=(CSDF=CSDF&i.) drop=Survival)
  %end; ;
  by &time.; drop STRATUM;
proc sort data = &out.; by &time.;
run;

/* No. at risk */
%if &atrisk. = 1 %then
  %km_atrisk(PLE, QEST, &stra., &time., &StraN., &atriskorder., &Base., &Step., &Size., &label., atrisk);
/* Censor extension */
%if &censEXT. = 1 %then
  %km_censEXT(&out., &time., &StraN., &cHeight., &cWidth., cens);

%if %eval(&censEXT.+&atrisk.) ^= 0 %then %do;
data &anno.;
  length when $1. style $30.;
  set
    %if &censEXT. = 1 %then cens;
    %if &atrisk. = 1 %then atrisk; ;
 retain when 'A' style &afont.;
run;
%end;

proc datasets lib = work;
  delete CS PLE QEST
    %if &censEXT. = 1 %then cens;
    %if &atrisk. = 1 %then atrisk; ;
run; quit;
%mend km_data;



/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/* censor extension */
%let color1 = 'cx5ECD22';
%let color2 = 'cxFAA55C';
%let color3 = 'cxE04251';
%let color4 = 'cx75B5E1';

%macro km_censEXT(
  data, time, StraN, cHeight, cWidth, out
);
data _null_;
  set &data.;
  call symput("exht", &cHeight. * (1 - min(of CSDF1 - CSDF&StraN.)) );
data &out.;
  length color function $8.;
  retain xsys ysys '2' size &cWidth.; set &data.;
  %do i = 1 %to &StraN.;
    if CSDF&i. ^= . then do;
      color = &&color&i.;
      function='move'; x=&time; y=CSDF&i.; output;
      function='draw'; x=&time; y=CSDF&i. + &exht.; output;
    end;
  %end;
  keep xsys ysys size color function x y;
run;
%mend km_censEXT;



/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/* No. of at risk */
%macro km_atrisk(
  data, est, stra, time, StraN, order,
  base, step, size, label, out
);
data &out.;
  set &data.; by STRATUM &time.;
  do x = &order.;
    if &time. <= x then do; leave; end;
  end;
data &out.;
  length color $10.; set &out.; by STRATUM x;
  retain xsys '2' ysys '1' color 'black' position '3' function 'label' size &size.;
  %do i = &StraN. %to 1 %by -1;
    if STRATUM = &i. then y = (&base. + &step. * (&StraN. - 1)) - &step. * (&i. - 1);
  %end;
  text = cat(Left);
  if last.x then output;
  keep xsys ysys color position size function x y text STRATUM;
data dummy;
  retain xsys '2' ysys '1' color 'black' position '3' function 'label' size &size.;
  do STRATUM = 1 to &StraN.;
  do x = &order.;
    %do i = &StraN. %to 1 %by -1;
      if STRATUM = &i. then y = (&base. + &step. * (&StraN. - 1)) - &step. * (&i. - 1);
    %end;
    output;
  end; end;
data &out.;
  merge dummy &out.; by STRATUM x;
  format xtext $20.; retain xtext '';
  if text ^= "" then xtext = text;
  else text = xtext;
data Label;
  length color function $8. text $100.;
  retain xsys '2' ysys '1' color 'black' position '3' function 'label' size &size.;
  x = 0; y = &base. + &step. * &StraN.; text = &label.; output;
data &out.; set &out. dummy Label;
proc datasets lib = work; delete dummy Label;
run;
%mend km_atrisk;
